// gr2pictlet.java
// save Java graphics to PICT format
// based on Jlpr (C) 1996 E.J.Friedman-Hill and Sandia National Labs

/**
Prints/saves applets using the Gr2PICT PICT Graphics context
@author Don Gilbert, 1996
 */
package dclap;

import java.applet.Applet;
import java.awt.*;
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.util.Enumeration;


public class Gr2PICTlet extends Applet implements Runnable {
    String kTitle = "Java graphics to PICT";
    String kDefaultFile = "myapplet.pict";
    String host, path, file;
    int port = 80;
    String fPrintLabel = "Generate PICT";
    Button fPrintButt,fRefreshButt;
    Choice appletChoice;
    TextField fTextField;
    Checkbox fFileSaveCheck, fNetSaveCheck;
    CheckboxGroup saveMethod;
    TestDraw testd;
    PrintMenu fPrintMenu;
    Thread menuThread, printThread;
    Frame fWindow;
    Container fCont;
    boolean isApp = false;


    public static void main(String argv[]) {
        Gr2PICTlet app = new Gr2PICTlet();
        app.initApp(argv);
    }

    public Gr2PICTlet() {
    }

    public String getAppletInfo() {
        return "Gr2PICTlet by Don Gilbert, based on Jlpr by E.J.Friedman-Hill";
    }


    public String[][] getParameterInfo() {
        String[][] info = {
            {"LABEL", "string", "Displayed on print button"},
            {"HOST", "string", "Internet host where file echoer lives"},
            {"PORT", "integer", "Internet port where file echoer lives"},
            {"PATH", "string", "Path of file echoer (e.g., /cgi-bin/echo"}
        };
        return info;
    }

    public void init() {
        isApp = false;
        fCont = this;
        init(null);
        initComponents();
    }

    public void initApp(String[] args) {
        isApp = true;
        fWindow = new AppFrame(this, kTitle);
        Panel pp = new Panel();
        fCont = pp;
        fWindow.setLayout(new BorderLayout());
        fWindow.add("Center", pp);
        init(args);
        initComponents();
        fWindow.pack();
        fWindow.resize(350, 200);
        fWindow.show();
    }

    public void init(String[] args) {
        if (!isApp)
            try {
                path = getParameter("PATH");
                fPrintLabel = getParameter("LABEL");
                host = getParameter("HOST");
                String sport = getParameter("PORT");
                if (sport != null) port = Integer.parseInt(sport);
            } catch (NullPointerException nullex) {
                showStatus("Missing parameters");
                System.out.println(nullex);
            }
        if (path == null) path = "/cgi-bin/echopict"; // hack
    }


    void initComponents() {
        fCont.setLayout(new BorderLayout());

        Panel np = new Panel();
        np.setLayout(new FlowLayout());
        fCont.add("South", np);
        saveMethod = new CheckboxGroup();
        np.add(fNetSaveCheck = new Checkbox("Use Net", saveMethod, !isApp));
        fNetSaveCheck.setBackground(Color.white);
        np.add(fFileSaveCheck = new Checkbox("Use File:", saveMethod, isApp));
        fFileSaveCheck.setBackground(Color.white);
        np.add(fTextField = new TextField(kDefaultFile));
        fTextField.setBackground(Color.white);
        np.add(new Checkbox("Check", null, true));

        np = new Panel();
        np.setLayout(new FlowLayout());
        fCont.add("Center", np);
        np.add(testd = new TestDraw());

        np = new Panel();
        np.setLayout(new FlowLayout());
        fCont.add("North", np);
        np.add(fPrintButt = new Button(fPrintLabel));
        fPrintButt.setBackground(Color.white);
        if (!isApp) {
            np.add(fRefreshButt = new Button("Refresh"));
            fRefreshButt.setBackground(Color.white);
        }
        np.add(appletChoice = new Choice());
        appletChoice.addItem(getClass().getName());
        appletChoice.select(0);
        fPrintMenu = new PrintMenu(this);
    }


    /**
     * Start the applet by forking a thread
     * which will look for other applets -- note that start()
     * doesn't call run() for this class!
     */
    public void start() {
        if (menuThread == null) {
            menuThread = new Thread(fPrintMenu);
            menuThread.start();
        }
    }

    public void stop() {
        if (menuThread != null) {
            menuThread.stop();
            menuThread = null;
        }
        if (printThread != null) {
            printThread.stop();
            printThread = null;
        }
    }


    public void showStatus(String msg) {
        if (!isApp) super.showStatus(msg);
        System.out.println(msg);
    }


    private Component printWhat() {
        if (isApp)
            return fCont;
        else {
            // what applet type is selected in choice?
            String type = appletChoice.getSelectedItem().trim();
            // Find an applet of this type
            Enumeration en = getAppletContext().getApplets();
            while (en.hasMoreElements()) {
                Object a = en.nextElement();
                if (a == null) continue;
                String atype = a.getClass().getName();
                if (type.equals(atype))
                    return (Component) a;
            }
            return null;
        }
    }

    public void run() {
        Component printThis = printWhat();
        if (printThis == null) return;
        //SavePICT saver= new SavePICT( this);

        String data;
        try {
            Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
            ByteArrayOutputStream baos = new ByteArrayOutputStream(10000);
            //saver.saveAsPict( printThis, baos);
            data = baos.toString();
        } catch (Throwable t) {
            showStatus("Error during rendering.");
            System.out.println(t);
            return;
        }

        boolean done = false;
        if (saveMethod.getCurrent() == fFileSaveCheck) {
            file = fTextField.getText();
            if (file == null || file.length() == 0) {
                file = kDefaultFile;
                fTextField.setText(file);
            }
            //done= saver.saveToFile( file, data);
        }
        if (!done) {
            if (host == null) {
                try {
                    URL baseu = getCodeBase();
                    host = baseu.getHost();
                    port = baseu.getPort();
                } catch (Exception ex) {
                    showStatus("Missing host");
                    System.out.println(ex);
                }
            }
            //saver.saveThruServer( host, port, path, data);
        }
    }


    public boolean action(Event e, Object arg) {
        if (e.target == appletChoice) {
            return true;
        } else if (e.target == fPrintButt) {
            if (printThread == null || !printThread.isAlive()) {
                printThread = new Thread(this);
                printThread.start();
                return true;
            }
        } else if (e.target == fRefreshButt) {
            if (menuThread != null) {
                menuThread.stop();
                menuThread = null;
            }
            start();
            return true;
        }
        return false;
    }

}

class PrintMenu implements Runnable {
    Gr2PICTlet p;
    private static int exceptionCount = 0;

    public PrintMenu(Gr2PICTlet p) {
        this.p = p;
    }

    private boolean findInChoice(String s, Choice c) {
        for (int i = 0; i < c.countItems(); i++)
            if (c.getItem(i).trim().equals(s))
                return true;
        return false;
    }

    public void run() {
        Choice c = p.appletChoice;
        Object a;
        String type;
        try {
            Enumeration e = p.getAppletContext().getApplets();

            // put all applets on the menu.
            while (e.hasMoreElements()) {
                a = e.nextElement();
                if (a == null) continue;
                type = a.getClass().getName();
                if (!findInChoice(type, c))
                    c.addItem(type);
            }
        } catch (Throwable ex) {
            // if anything bad happens, wait a bit, then try again, five times
            ++exceptionCount;
            try {
                System.out.println(ex);
                Thread.currentThread().sleep(200);
            } catch (InterruptedException ex2) {
                // whatever
            }
            if (exceptionCount < 5)
                run();
        }
        c.layout();
        exceptionCount = 0;
    }

}


class TestDraw extends Canvas {

    TestDraw() {
        setBackground(Color.white);
        resize(minimumSize());
    }

    public Dimension minimumSize() {
        return new Dimension(200, 130);
    }

    public void paint(Graphics g) {
        int x, y, w, h;
        Dimension d = size();
        x = 0;
        y = 0;
        w = d.width - 1;
        h = d.height - 1;

        g.drawRect(x, y, w, h);

        x += 20;
        y += 20;
        w *= 0.5;
        h *= 0.6;
        g.setFont(new Font("Times", Font.PLAIN, 12));
        g.drawString("TestDraw", x, y);
        g.setColor(Color.red);
        g.fillRect(x, y, w, h);
        g.setColor(Color.black);
        g.drawLine(x, y, x + w, y + h);

        x += 20;
        y += 20;
        w *= 0.5;
        h *= 0.6;
        g.setColor(Color.green);
        g.fillOval(x, y, w, h);
        g.setFont(new Font("Courier", Font.BOLD, 14));
        g.drawString("TestDraw", x, y);

        x += 20;
        y += 20;
        w *= 0.5;
        h *= 0.6;
        g.setColor(Color.blue);
        g.fillRoundRect(x, y, w, h, 10, 10);

        g.setFont(new Font("Helvetica", Font.ITALIC, 18));
        g.drawString("TestDraw", x, y);
    }
}


