package diffcad;

import futils.Print;
import observers.ObservableDouble;

import java.awt.*;


class Camera extends Shape {
    // given a dilation parameter, return a valid point
    public Grating grating;

    ObservableDouble A = new ObservableDouble(4.8, "A:");
    ObservableDouble F = new ObservableDouble(3.6, "F:");
    boolean backwards = false;
    boolean auto_pan = false;
    boolean i3_negative = false;
    boolean r3_negative = false;
    boolean order_negative = false;


    Camera() {
        super();
        rotation = new ObservableDouble(45, "Rho:");
    }

    void center() {
        p1.x = grating.L / 2 - A.getValue() / 2;
        p1.y = -grating.L * F.getValue() / A.getValue();
        rotation.setValue(0);
    }

    double rho() {
        return degrees_to_radians(rotation.getValue());
    }

    void print() {
        Print.className(this);
        print("rho=\t" + rotation.getValue() + "\t");
        print("pc=\t");
        pc.print();
        print("\tA=\t" + A.getValue() + "\t");
        Print.ln("F=\t" + F.getValue());
        print_d();
    }

    void print_d() {
        System.out.println("Camera: d = " + compute_d());
    }

    double compute_d() {
        return -pc.y;
    }

    void update() {
        pc.x = p1.x + A.getValue() / 2.0;
        pc.y = p1.y;
        p2.x = p1.x + A.getValue();
        p2.y = p1.y;

        p3.x = p2.x;
        p3.y = p1.y - F.getValue();

        p4.x = p1.x;
        p4.y = p3.y;

        pb.x = pc.x;
        pb.y = p4.y;
        if (auto_pan) {
            rotation.setValue(radians_to_degrees(
                    Math.atan((pc.x - grating.L / 2) / -pc.y)));
        }
        System.out.println("Rho = Rotation = " + rotation.getValue());

        pointRotation(p1);
        pointRotation(p2);
        pointRotation(p3);
        pointRotation(p4);
        pointRotation(pb);
    }

    void change_config(double x, double y) {
        p1.x = x;
        p1.y = y;
        update();
    }

    void change_config(point t) {
        change_config(p1.x + t.x, p1.y + t.y);
    }


    double compute_n_lambda_on_p(double x3) {
        return Geometry.n.getValue() * Geometry.lambda.getValue() / compute_pitch(x3);
    }

    point x3(double t) {
        return x3_to_point(x_min(t));
    }

    double compute_pitch(double x3) {
        double displacementFromLeft = pc.y * Math.tan(compute_r_r(x3)) + pc.x;
        double t = displacementFromLeft / grating.L;
        // t is the dilation parameter, t =0..1;
        return grating.pitch(t);

    }

    // Given a position on the back of the camera, return a
    // point in object coordinates. From pb to right is positive
    // from pb to left is negative.
    // x3 ranges from - A/2 to A/2
    point x3_to_point(double x3) {
        Ray back_of_camera = new Ray();
        double t;

        if (x3 < -A.getValue() / 2) {
            System.out.println("x3 smaller than - A.getValue()/2, x3=" + x3);
        }
        if (x3 > A.getValue() / 2) {
            System.out.println("x3 bigger than   A/2 x3=" + x3);
        }
        // make a parameter which varies from 0..1
        t = (x3 + A.getValue() / 2) / A.getValue();

        back_of_camera.p1 = p4;
        back_of_camera.p2 = p3;
        return back_of_camera.dilation(t);

    }

    double x_max() {

        double foo;
        foo = -Math.tan(rho() + Math.atan(pc.x / pc.y)) * F.getValue();

        return foo;
    }

    // Hey, This works good, I luv a small program!
    point x_min_p() {
        return x3_to_point(x_min());
    }

    point x_max_p() {
        return x3_to_point(x_max());
    }

    void print_x() {
        System.out.println(
                "x_min =" + x_min() +
                " x_max = " + x_max());
    }

    double x_min() {
        double rho = rotation.getValue() * Pi_on_180;
        double foo;

        foo = -F.getValue() * Math.tan(rho + Math.atan((pc.x - grating.L) / pc.y));

        return foo;
    }


    double x_min(double t) {
        double rho = rotation.getValue() * Pi_on_180;
        double foo;

        foo = -F.getValue() * Math.tan(rho + Math.atan((pc.x - grating.L * t) / pc.y));

        return foo;
    }

    // return the angle in radians. For camera only.


//This only works for the camera object
    double compute_r_r(double x3) {
        return rho() + Math.atan(x3 / F.getValue());
    }


    double compute_r_d(double x3) {
        return radians_to_degrees(compute_r_r(x3));
    }

    point compute_g3(double x3) {
        return new point(
                pc.y * Math.tan(compute_r_r(x3)) + pc.x, 0);
    }

    double compute_asin_arg(double x3) {

        double r3;
        double sine_r;
        double n_l_on_p;
        double return_val;
        int order;
        r3 = compute_r_r(x3);
        sine_r = Math.sin(r3);
        order = 1;
        if (order_negative) {
            order = -1;
        }
        n_l_on_p = order * Geometry.lambda.getValue() / compute_pitch(x3);
        if (r3_negative) {
            sine_r = sine_r * -1;
        }
        return_val = n_l_on_p + sine_r;
        if (Math.abs(return_val) < 0.99)
            return return_val;
        else
            return 0;

    }

    double compute_i_d(double x3) {
        return radians_to_degrees(compute_i_r(x3));
    }

    double compute_i_r(double x3) {
        int sign_i3;
        sign_i3 = 1;
        if (i3_negative) {
            sign_i3 = -1;
        }
        return sign_i3 * Math.asin(compute_asin_arg(x3));
    }


    void draw(Graphics g) {
        pushGraphicsState(g);
        // p1 -pc- p2
        //  |      |
        // p4      p3
        //   - pb-
        // Mouse click comes in x1 and y1, upper left corner

        line(g, p1, p2);
        line(g, p2, p3);
        line(g, p3, p4);
        line(g, p4, p1);
        line(g, pc, pb);

        string(g, "PC", pc);
        popGraphicsState(g);
    }

}
