package diffcad;

import futils.Futil;
import futils.Print;
import observers.ObservableInt;

import java.awt.*;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Vector;


public final class Geometry {
    // prevent instantiation
    private Geometry() {
    }


    static public Grid grid = new Grid();

    static public ObservableInt n =
            new ObservableInt(1, "n:"); // order;
    static public ObservableInt lambda =
            new ObservableInt(632, "lambda:");
    static public ObservableInt step_size =
            new ObservableInt(20, "Step size:");
    static public double Pi_on_180 = Math.PI / 180;
    static public String APP_NAME = "DiffCAD";

    static public Shape camera = new Camera();
    static public diffcad.Shape grating = new Grating();
    static public diffcad.Shape wedge = new Wedge();
    static public diffcad.Shape laser = new Laser();
    static public diffcad.Shape lwl = new
            LaserWedgeLine(
                    (Laser) laser,
                    (Wedge) wedge);
    static public ObservableInt number_of_rays =
            new ObservableInt(10, "Number of Rays:");

    static public diffcad.Shape grating_target_line[] =
            new diffcad.Shape[number_of_rays.getValue()];
    static public diffcad.Shape camera_grating_line[] =
            new diffcad.Shape[number_of_rays.getValue()];

    static public double delta_t() {
        return 1 / (double) (number_of_rays.getValue() - 1);
    }

    public static void save() {
        FileOutputStream os = Futil.getWriteFileOutputStream();
        PrintStream output = new PrintStream(os);
        output.println("Data format is order dependent");
        Geometry.print(output);
        Futil.closeOutputStream(os);
    }

    static void printShapes(Vector v) {
        diffcad.Shape s;
        for (int i = 0; i < v.size(); i++) {
            s = (diffcad.Shape) v.elementAt(i);
            s.print();
        }
    } // printShapes

    static public void print(PrintStream out) {
        Print.setPrintStream(out);
        print();
        Print.setPrintStream(System.out);
    } // print

    static public void print() {
        laser.print();
        camera.print();
        wedge.print();
        grating.print();
        System.out.println("Lambda = " + lambda.getValue());
        System.out.println("s = " + s());
    } // print

    static public double s() {
        Laser l = (Laser) laser;
        return l.NG() + camera.pc.x;
    }

    static public void print_angles() {
        double t = 0;
        double x3,comp_p,p3,comp_r, r3,i3,r3r,i3r, dl, myi3r;
        double comp_i,comp_asin,my_asin,comp_nlp;
        double alpha;

        double my_nlp;
        alpha = laser.rotation.getValue();
        System.out.println("Alpha =" + alpha);
        System.out.print("comp_n_lambda_on_p\tmy_nlp\t");
        System.out.print("comp_i\tI3\t");
        System.out.print("Comp_p\tP3\tcomp_r\t R3\t ");
        System.out.print("Comp_arg_to_sin\t asin\t x3");
        System.out.print("\t DL\t nlambda_on_p\t");
        System.out.println("sin(r)\tnlambda_on_p -sin(r)\ti3r");

        Camera c = (Camera) camera;
        Grating g = (Grating) grating;

        for (int i = 0; i < number_of_rays.getValue(); i++) {
            p3 = g.pitch(t);
            CameraGratingLine cgl =
                    (CameraGratingLine) camera_grating_line[i];
            GratingTargetLine gtl =
                    (GratingTargetLine) grating_target_line[i];
            r3 = cgl.r_angle_d();
            i3 = cgl.i_angle_d(g.pitch(t));
            dl = gtl.target.p1.magnitude();
            x3 = c.x_min(t);
            my_nlp = n.getValue() * Geometry.lambda.getValue() / p3;
            my_asin = my_nlp - Math.sin(r3 * 180 / Math.PI);
            comp_p = c.compute_pitch(x3);
            comp_r = c.compute_r_d(x3);
            comp_i = c.compute_i_d(x3);
            comp_nlp = c.compute_n_lambda_on_p(x3);
            comp_asin = c.compute_asin_arg(x3);
            r3r = cgl.r_angle_r();
            i3r = cgl.i_angle_r(g.pitch(t));
            Print.d(comp_nlp);
            Print.d(my_nlp);
            Print.d(comp_i);
            Print.d(i3);
            Print.d(comp_p);
            Print.d(p3);
            Print.d(comp_r);
            Print.d(r3);
            Print.d(comp_asin);
            Print.d(my_asin);

            Print.d(x3);
            Print.d(dl);
            Print.d(n.getValue() * Geometry.lambda.getValue() / p3);
            myi3r = n.getValue() * Geometry.lambda.getValue() / p3 - Math.sin(r3r);

            Print.d(Math.sin(r3r));

            // print n*lambda/p - sin(r);
            Print.d(i3r);
            System.out.println();

            t = t + delta_t();
        }
    } // print_angles

    static public void init() {
        set_up_colors();
        set_up_camera();
        set_up_grating();
        set_up_laser_wedge();
        laser.rotation.setName("Alpha:");
        allocate_ray_memory();
        compute_rays();
    }

    static public void add_elements(Vector v) {
        for (int i = 0; i < number_of_rays.getValue(); i++) {
            v.addElement(grating_target_line[i]);
            v.addElement(camera_grating_line[i]);
        }
        v.addElement(camera);
        v.addElement(grating);
        v.addElement(wedge);
        v.addElement(laser);
        v.addElement(lwl);
    } // add_elements

    static public void set_up_grating() {
        Grating g = (Grating) grating;
        g.p1 = new point(0, 0);
        g.p2 = new point(0, g.L);
    } // set_up_grating

    static public void allocate_ray_memory() {
        LaserWedgeLine lwl2 =
                (LaserWedgeLine) lwl;
        for (int i = 0; i < number_of_rays.getValue(); i++) {

            grating_target_line[i] =
                    new GratingTargetLine(lwl2);

            camera_grating_line[i] =
                    new CameraGratingLine();

        }
    } // allocate_ray_memory


    static public void compute_rays() {
        double t = 0;
        double i3; // incident angle upon the grating.
        double x3; // displacement away from the center of the arrayboard.
        Camera c = (Camera) camera;
        for (int i = 0; i < number_of_rays.getValue(); i++) {

            x3 = c.x_min(t);
            // get a point on the arrayboard
            camera_grating_line[i].p2 = c.x3_to_point(x3);
            // get a point on the grating.
            camera_grating_line[i].p1 = c.compute_g3(x3);

            // use the same grating point to fire a ray to the
            // target.
            grating_target_line[i].p1 = c.compute_g3(x3);

            // Get the incident angle at that point on the grating
            i3 = c.compute_i_r(x3);
            // Fire the ray thata way
            grating_target_line[i].point_toward(i3);


            t = t + delta_t();
        }
    } // compute_rays

    static public void set_up_laser_wedge() {


        double alpha;


        laser.p1.x = 11.167;
        laser.p1.y = -19.756;

        // laser.rotation = 57.848/2;
        laser.rotation.setValue(60);
        laser.rotation.setName("Alpha:");

        // alpha is in radians
        alpha = laser.rotation.getValue() * Pi_on_180;

        //laser.rotation = 85;
        Laser l = (Laser) laser;
        Wedge w = (Wedge) wedge;
        l.update();

        w.p1.x = -1270 * Math.sin(alpha) + l.pc.x;
        w.p1.y = 1270 * Math.cos(alpha) + l.pc.y;
        w.laser = l;
        // cast shape to LaserWedgeLine
        LaserWedgeLine lwl2 = (LaserWedgeLine) lwl;
        lwl2.laser = l;
        lwl2.wedge = w;

    } // set_up_laser_wedge

    static public void set_up_colors() {
        camera.setColor(Color.orange);
        grating.setColor(Color.pink);
        wedge.setColor(Color.blue);
        laser.setColor(Color.red);
    } // set_up_colors

    static public void set_up_camera() {

        /* This worked OK! */
        Camera c = (Camera) camera;
        Geometry.lambda = lambda;
        c.grating = (Grating) grating;
        Grating g = (Grating) grating;
        double offset = 80;
        c.p1.x = offset + g.L / 2 + c.A.getValue() / 2;
        c.p1.y = -offset * .52 - g.L * c.F.getValue() / c.A.getValue();
        c.rotation.setValue(
                -(180 / Math.PI) * Math.atan(c.p1.y / (c.p1.x - g.L / 2))
                - 32.67);
        c.update();
    } // set_up_camera

}