package ipl;

import vs.*;
import futils.Futil;
import diffcad.point;

import java.awt.*;
import java.awt.image.PixelGrabber;
import java.io.FileOutputStream;
import java.io.PrintStream;


public class ProcessPlane extends PixelPlane {

    private FFTPlane fftp;
    private HLS ConvHLS;
    private HSV ConvHSV;
    private CMY ConvCMY;
    private IYQ ConvIYQ;

    private ImageUtils iutils = new ImageUtils();

    public ProcessPlane(double x_, double y_) {
        super(x_, y_);
    }

    public ProcessPlane(int x_, int y_) {
        super(x_, y_);
    }

    public int getGray(int i) {
        return
                (getRed(i) + getBlue(i) + getGreen(i)) / 3;
    }

    public int getMinIntensity() {
        int minI = 99999;
        for (int i = 0; i < pels.length; i++)
            if (minI > getGray(i))
                minI = getGray(i);
        return minI;
    }

    public int getMaxIntensity() {
        int maxI = -99999;
        for (int i = 0; i < pels.length; i++)
            if (maxI < getGray(i))
                maxI = getGray(i);
        return maxI;
    }


    private int xfer(double f) {
        double sf = 31.875 / Math.log(2.0);
        return ((int) (sf * Math.log(f + 1)));
    }


    public void cornergray() {
        int r,g,b,a;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = (x * y) % 256;
                setPixel(x, y, r, r, r, 255);
            }
    }

    public void diagGray() {
        int r,g,b,a;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = (x + y) % 255;
                setPixel(x, y, r, r, r, 255);
            }
    }

    public void lines(Rectangle r, int increment) {
        int x1 = 0;
        int y1 = 0;
        int x2 = r.width;
        int y2 = r.height;
        dot(x2 / 2, y2 / 2, 2 * x2);
        for (int x = x1; x < x2; x += increment) {
            for (int y = y1; y < y2; y++) {
                setPixel(x, y, 255, 255, 255, 255);
            }
        }

    }

    public void multimage(ProcessPlane pp) {
        int x1 = 0;
        int y1 = 0;
        int x2 = getWidth();
        int y2 = getHeight();
        double r, g, b;
        for (int x = x1; x < x2; x++) {
            for (int y = y1; y < y2; y++) {
                r = pp.getRed(x, y) / 255.0;
                g = pp.getGreen(x, y) / 255.0;
                b = pp.getBlue(x, y) / 255.0;
                r *= getRed(x, y);
                g *= getGreen(x, y);
                b *= getBlue(x, y);
                setPixel(x, y, (int) r, (int) g, (int) b, 255);
            }
        }
    }

    public void chirp(Rectangle r, int increment, double rate) {
        int x1 = 0;
        int y1 = 0;
        int x2 = r.width;
        int y2 = r.height;
        int i = 255;
        double dx = (double) increment;
        dot(x2 / 2, y2 / 2, 2 * x2);
        for (double x = x1; x < x2; x += dx) {
            for (int y = y1; y < y2; y++) {
                setPixel((int) x, y, i, i, i, 255);
            }
            dx += rate;
        }

    }

    public void dot(int xc, int yc, int r) {

        int x1 = 0;
        int y1 = 0;
        int x2 = getWidth();
        int y2 = getHeight();
        float r2 = r * r;
        for (int x = x1; x < x2; x++)
            for (int y = y1; y < y2; y++) {
                if (((x - xc) * (x - xc) + (y - yc) * (y - yc)) < r2)
                    setPixel(x, y, 0, 0, 0, 255);
                else
                    setPixel(x, y, 255, 255, 255, 255);
            }
    }

    public ProcessPlane scale(int scale) {
        int d = 0;

        int width = getWidth();
        int height = getHeight();
        ProcessPlane pp =
                new ProcessPlane(
                        width * scale, height * scale);
        for (int y = 0; y < height; y++)
            for (int j = 0; j < scale; j++)
                for (int x = 0; x < width; x++)
                    for (int l = 0; l < scale; l++)
                        pp.pels[d++] = getPixel(x, y);
        return pp;
    }

    public void threshold(int konst) {
        int r,g,b,a;

        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = getRed(x, y);
                g = getGreen(x, y);
                b = getBlue(x, y);
                a = getAlpha(x, y);
                if ((r + g + b) > (konst * 3)) {
                    setPixel(x, y, 255, 255, 255, a);
                } else {
                    setPixel(x, y, 0, 0, 0, a);
                }

            }
    }

    public void edgetoxy() {
        FileOutputStream fos = Futil.getFileOutputStream();
        PrintStream ps = new PrintStream(fos);
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                if (getRed(x, y) == 255) {
                    ps.println(x + " " + y);
                }
                ;
            }
        }
        Futil.closeOutputStream(fos);
    }

    public ProcessPlane edge() {
        ProcessPlane pp = new ProcessPlane(getWidth(), getHeight());

        int r,a;
        int average; // average intensity
        for (int y = 0; y < getHeight(); y++)
            new_line: {
                average = 0;
                for (int x = 0; x < getWidth(); x++)
                    average = average + getRed(x, y);
                average = average / getWidth();
                for (int x = 0; x < getWidth(); x++) {
                    r = getRed(x, y);
                    a = getAlpha(x, y);
                    if (r > average) {
                        int start = x;
                        for (int i = x; i < getWidth(); i++) {
                            r = getRed(i, y);
                            if (r < average) {
                                pp.setPixel(
                                        start + (i - start) / 2,
                                        y, 255, 255, 255, a);
                                break new_line;
                            }
                        }
                    }
                }
            }
        return pp;
    }

    public void linearComb(double konst, double ak) {
        double r,g,b,a;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = ak + konst * getRed(x, y);
                g = ak + konst * getGreen(x, y);
                b = ak + konst * getBlue(x, y);
                a = getAlpha(x, y);
                setPixel(x, y, (int) r, (int) g, (int) b, (int) a);
            }

    }

    public void Subimage(ProcessPlane pp) {
        double r,g,b,a;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = getRed(x, y) - pp.getRed(x, y);
                g = getGreen(x, y) - pp.getGreen(x, y);
                b = getBlue(x, y) - pp.getBlue(x, y);
                a = getAlpha(x, y);
                setPixel(x, y, (int) r, (int) g, (int) b, (int) a);
            }

    }


    public void makeGray() {
        int r,g,b,a;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = getRed(x, y);
                a = getAlpha(x, y);
                setPixel(x, y, r, r, r, a);
            }
    }


    public void randResample() {
        int r,g,b,a;
        int randx, randy;
        for (int y = 0; y < getHeight() - 2; y++)
            for (int x = 0; x < getWidth() - 2; x++) {
                r = getRed(x, y);
                g = getGreen(x, y);
                b = getBlue(x, y);
                a = getAlpha(x, y);
                randx = (int) (Math.random() * 2);
                randy = (int) (Math.random() * randx);
                setPixel(x + randx, y + randy, r, g, b, a);
            }
    }


    public void shadow() {
        int r,g,b,a;
        for (int y = 0; y < getHeight() - 2; y++)
            for (int x = 0; x < getWidth() - 2; x++) {
                r = getRed(x, y);
                g = getGreen(x, y);
                b = getBlue(x, y);
                a = getAlpha(x, y);
                r = r + (127 - getRed(x + 2, y + 2));
                g = g + (127 - getGreen(x + 2, y + 2));
                b = b + (127 - getBlue(x + 2, y + 2));
                setPixel(x, y, r, g, b, a);
            }
    }

    public void fft() {
        System.out.println("Running the FFT...");
        fftp = new FFTPlane(this);
        fftp.fft();
    }

    public void multFFT(ProcessPlane pp_) {
        if (fftp == null) {
            fft();
        } else {
            fftp.mult(pp_);
        }
    }

    public void noiseFFT(double t) {
        fftp.threshold(t);

    }

    public void ifft() {
        System.out.println("Running the iFFT...");
        if (fftp == null) {
            System.out.println("You must take the FFT first!");
        } else
            fftp.ifft();
    }

    public void brighten(double sf) {
        rgb2hls();
        ConvHLS.brighten(sf);
        hls2rgb();
    }

    public void equalize() {
        rgb2hls();
        ConvHLS.equalize();
        hls2rgb();
    }

    public void equalizedep() {
        int r,g,b,a;
        int minIntensity = getMinIntensity();
        int maxIntensity = getMaxIntensity();
        System.out.println("min and max intensity are");
        System.out.println(minIntensity + " " + maxIntensity);
        int sf = maxIntensity - minIntensity;
        for (int y = 0; y < getHeight(); y++)
            for (int x = 0; x < getWidth(); x++) {
                r = getRed(x, y);
                g = getGreen(x, y);
                b = getBlue(x, y);
                a = getAlpha(x, y);
                r = xfer(r);
                g = xfer(g);
                b = xfer(b);
                setPixel(x, y, r, g, b, a);
            }
    }

    public void autoScale() {
        rgb2hls();
        ConvHLS.autoScale();
        hls2rgb();
    }

    public void xform(Mat3 transform) {
        int w = getWidth();
        int h = getHeight();
        int xc = w / 2;
        int yc = h / 2;

        ProcessPlane pp = new ProcessPlane(w, h);

        transform = transform.transpose();

        double p[] = new double[3];
        int pixel;
        int xp, yp;

        for (int x = -xc; x < xc; x++)
            for (int y = -yc; y < yc; y++) {
                p = transform.multiply(x, y, 1);
                xp = (int) p[0] + xc;
                yp = (int) p[1] + yc;
                if ((xp < w) && (yp < h) && (xp >= 0) && (yp >= 0)) {
                    pixel = getPixel(xp, yp);
                    pp.setPixel(x + xc, y + yc, pixel);
                }
            }
        pels = pp.pels;
    }

    public void xformfeedback(Mat3 transform) {
        int w = getWidth();
        int h = getHeight();
        int xc = w / 2;
        int yc = h / 2;

        double p[] = new double[3];
        int pixel;
        int xp, yp;
        transform = transform.transpose();
        for (int x = -xc; x < xc; x++)
            for (int y = -yc; y < yc; y++) {
                p = transform.multiply(x, y, 1);
                xp = (int) p[0] + xc;
                yp = (int) p[1] + yc;
                if ((xp < w) && (yp < h) && (xp >= 0) && (yp >= 0)) {
                    pixel = getPixel(xp, yp);
                    setPixel(x + xc, y + yc, pixel);
                }
            }
    }

    public void turn(double degrees) {
        double pion180 = Math.PI / 180.0;
        double theta = degrees * pion180;
        xform(Mat3.rotation(theta));
    }

    public void turnfb(double degrees) {
        double pion180 = Math.PI / 180.0;
        double theta = degrees * pion180;
        xformfeedback(Mat3.rotation(theta));
    }

    public void zoom(double percentage) {
        xform(Mat3.scaling(new point(percentage, percentage)));
    }

    public void zoomfeedback(double percentage) {
        xformfeedback(Mat3.scaling(new point(percentage, percentage)));
    }

    public void shearx(double shx) {
        xform(Mat3.shear(new point(shx, 0)));
    }

    public void sheary(double shy) {
        xform(Mat3.shear(new point(0, shy)));
    }

    public void shearxfb(double shx) {
        xformfeedback(Mat3.shear(new point(shx, 0)));
    }

    public void shearyfb(double s) {
        zoomfeedback(s);
        Mat3 t = Mat3.rotation(90 * Math.PI / 180);
        zoomfeedback(s);
        xformfeedback(t);
        zoomfeedback(s);
        xformfeedback(t);
        zoomfeedback(s);
        xformfeedback(t);
        zoomfeedback(s);
        xformfeedback(t);
    }

    public void rgb2iyq() {
        ConvIYQ = new IYQ(this);
        pels = ConvIYQ.fromRGB();
    }

    public void iyq2rgb() {
        if (ConvIYQ != null)
            pels = ConvIYQ.toRGB();
        else
            System.out.println("Perform RGB2IYQ first!");
    }

    public void rgb2cmy() {
        ConvCMY = new CMY(this);
        pels = ConvCMY.fromRGB();
    }

    public void cmy2rgb() {
        if (ConvCMY != null)
            pels = ConvCMY.toRGB();
        else
            System.out.println("Perform RGB2CMY first!");
    }

    public void rgb2hls() {
        // The HLS contains float results
        // from the conversion
        ConvHLS = new HLS(this);

        pels = ConvHLS.fromRGB();
    }

    public void hls2rgb() {
        if (ConvHLS != null)
            pels = ConvHLS.toRGB();
        else
            System.out.println("Perform rgb2hls first!");
    }

    public void rgb2hsv() {
        ConvHSV = new HSV(this);
        pels = ConvHSV.fromRGB();
    }


    public void hsv2rgb() {
        if (ConvHSV != null)
            pels = ConvHSV.toRGB();
        else
            System.out.println("Perform rgb2HSV first!");
    }

    public void negate() {
        int r,g,b,a;
        int L = pels.length;
        for (int i = 0; i < L; i++) {
            r = getRed(i);
            g = getGreen(i);
            b = getBlue(i);
            a = getAlpha(i);
            setPixel(i, 255 - r, 255 - g, 255 - b, a);
        }
    }

    public static ProcessPlane openPPM(String fileName) {
        System.out.println("PPM image selected");
        ProcessPlane pp = null;
        int pels[];
        try {
            pels = ReadPPM.DoIt(fileName);
            pp = new ProcessPlane(ReadPPM.width, ReadPPM.height);
            pp.pels = pels;
            System.out.println(ReadPPM.width + "x" + ReadPPM.height);

        } catch (Exception e) {
            System.out.println("Read PPM Exception!");
        }
        return pp;
    }

    public void grabPels(Image image) {
        int w = getWidth();
        int h = getHeight();

        System.out.println("Starting pixel grabber on a " +
                w + "x" + h);
        int x = 0;
        int y = 0;
        int off = 0;
        PixelGrabber pg =
                new PixelGrabber(image, x, y, w, h, pels, off, w);
        System.out.println("grabbing...");
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
            System.out.println("Interrupted waiting for pixels");
        }
    }

    public static ProcessPlane openGIF(String fileName) {
        Frame f = new Frame();
        ProcessPlane pp;
        Image image = Toolkit.getDefaultToolkit().getImage(
                fileName);
        waitForImage(f, image);
        pp = new ProcessPlane(image.getWidth(f), image.getHeight(f));
        pp.grabPels(image);
        return pp;
    }

    public static ProcessPlane image2ProcessPlane(Image image) {
        Frame f = new Frame();
        ProcessPlane pp;
        waitForImage(f, image);
        pp = new ProcessPlane(image.getWidth(f), image.getHeight(f));
        pp.grabPels(image);
        return pp;
    }

    public void writePPM() {
        ColorUtils CU = new ColorUtils();

        try {
            WritePPM.DoIt(
                    CU.getRedArray(pels),
                    CU.getGreenArray(pels),
                    CU.getBlueArray(pels),
                    getWidth(), getHeight(),
                    Futil.getWriteFileName());
        } catch (Exception e) {
            System.out.println("Save PPM Exception!");
        }
    }

    public void writePPM(String fn) {
        ColorUtils CU = new ColorUtils();

        try {
            WritePPM.DoIt(
                    CU.getRedArray(pels),
                    CU.getGreenArray(pels),
                    CU.getBlueArray(pels),
                    getWidth(), getHeight(),
                    fn);
        } catch (Exception e) {
            System.out.println("Save PPM Exception!");
        }
    }

    public void saveGIF() {

        try {
            WriteGIF.DoIt(
                    makeImage(),
                    Futil.getWriteFileName());
        } catch (Exception e) {
            System.out.println("Save GIF Exception!");
        }

    }

    public void saveGIF(String fn) {

        try {
            WriteGIF.DoIt(
                    makeImage(), fn);
        } catch (Exception e) {
            System.out.println("Save GIF Exception!");
        }

    }

    private static void waitForImage(Component component,
                                     Image image) {
        MediaTracker tracker = new MediaTracker(component);
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll();
            tracker.waitForID(0);
            if (tracker.checkID(0))
                System.out.println("The image checked as loaded");
            else
                System.out.println("Load failure!");
        } catch (InterruptedException e) {
            Assert.notNull(null);
        }
    }

    public ProcessPlane roberts() {
        int Gi = 0;
        int Gj = 0;
        int G = 0;
        int max_G = 0;
        int z5 = 0;
        int z6 = 0;
        int z8 = 0;
        int z9 = 0;

        //	The mask used to compute Gi at a center point of a 3x3 region
        //	 1	 0			z5	z6
        //	 0	-1 ,where	z8	z9

        //	The mask used to compute Gj at a center point of a 3x3 region
        //
        //	 0	1			z5	z6
        //	-1	0 ,where	z8	z9

        //	The 3x3 region
        //	z1	z2	z3
        //	z4	z5	z6
        //	z7	z8	z9

        // To approximate the gradient G of a pixel
        //
        //	G = |z5 - z9| + |z6 - z8|, or
        //	G = |Gi| + |Gj|

        ProcessPlane temp_pp = new ProcessPlane(getWidth(), getHeight());
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                temp_pp.setPixel(x, y, 0, 0, 0, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                if ((x < getWidth() - 1) && (y < getHeight() - 1)) {
                    z5 = getPixel(x, y);
                    z9 = getPixel(x + 1, y + 1);
                    z6 = getPixel(x + 1, y);
                    z8 = getPixel(x, y + 1);
                    Gi = z5 - z9;
                    Gj = z6 - z8;
                } else if ((x == getWidth() - 1) && (y < getHeight() - 1)) {
                    z5 = getPixel(x, y);
                    z8 = getPixel(x, y + 1);
                    Gi = z5;
                    Gj = z8;
                } else if ((x < getWidth() - 1) && (y == getHeight() - 1)) {
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    Gi = z5;
                    Gj = z6;
                } else if ((x == getWidth() - 1) && (y == getHeight() - 1)) {
                    z5 = getPixel(x, y);
                    Gi = getPixel(x, y);
                    Gj = getPixel(x, y);
                }
                G = Math.abs(Gi) + Math.abs(Gj);
                if (G > max_G)
                    max_G = G;
                temp_pp.setPixel(x, y, (int) G, (int) G, (int) G, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                G = temp_pp.getPixel(x, y);
                temp_pp.setPixel(x, y, (int) (G / max_G) * 255, (int) (G / max_G) * 255,
                        (int) (G / max_G) * 255);
            }
        }
        return temp_pp;
    }

    public ProcessPlane sobel() {
        int Gx = 0;
        int Gy = 0;
        int G = 0;
        int max_G = 0;
        int z1 = 0;
        int z2 = 0;
        int z3 = 0;
        int z4 = 0;
        int z6 = 0;
        int z7 = 0;
        int z8 = 0;
        int z9 = 0;

        //	The mask used to compute Gx at a center point of a 3x3 region
        //
        //	-1	-2	-1			z1	z2	z3
        //	 0	 0	 0			z4	z5	z6
        //	 1	 2	 1 ,where	z7	z8	z9

        //	The mask used to compute Gy at a center point of a 3x3 region
        //
        //	-1	0	1			z1	z2	z3
        //	-2	0	2			z4	z5	z6
        //	-1	0	1 ,where	z7	z8	z9

        // To approximate the gradient G of a pixel
        //
        //	Gx = (z7 + 2*z8 + z9) - (z1 + 2*z2 + z3)
        //	Gy = (z3 + 2*z6 + z9) - (z1 + 2*z4 + z7)
        //	G = |Gx| + |Gy|

        ProcessPlane temp_pp = new ProcessPlane(getWidth(), getHeight());
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                temp_pp.setPixel(x, y, 0, 0, 0, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                if ((x > 0) && (x < getWidth() - 1) && (y > 0) && (y < getHeight() - 1)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z4 = getPixel(x - 1, y);
                    z6 = getPixel(x + 1, y);
                    Gx = z7 + 2 * z8 + z9 - z1 - 2 * z2 - z3;
                    Gy = z3 + 2 * z6 + z9 - z1 - 2 * z4 - z7;
                } else if ((x == 0) && (y == 0)) {
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z6 = getPixel(x + 1, y);
                    Gx = 2 * z8 + z9;
                    Gy = 2 * z6 + z9;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z6 = getPixel(x + 1, y);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + 2 * z8 + z9;
                    Gy = 2 * z6 + z9 - 2 * z4 - z7;
                } else if ((x == getWidth() - 1) && (y == 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + 2 * z8;
                    Gy = -2 * z4 - z7;
                } else if ((x == 0) && (y > 0) && (y < getHeight() - 1)) {
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    Gx = 2 * z8 + z9 - 2 * z2 - z3;
                    Gy = z3 + 2 * z6 + z9;
                } else if ((x == 0) && (y == getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    Gx = -2 * z2 - z3;
                    Gy = z3 + 2 * z6;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == getHeight() - 1)) {
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    z4 = getPixel(x - 1, y);
                    Gx = -z1 - 2 * z2 - z3;
                    Gy = z3 + 2 * z6 - z1 - 2 * z4;
                } else if ((x == getWidth() - 1) && (y == getHeight() - 1)) {
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    Gx = -z1 - 2 * z2;
                    Gy = -z1 - 2 * z4;
                } else if ((x == getWidth() - 1) && (y < getHeight() - 1) && (y > 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + 2 * z8 - z1 - 2 * z2;
                    Gy = -z1 - 2 * z4 - z7;
                }
                G = Math.abs(Gx) + Math.abs(Gy);
                if (G > max_G)
                    max_G = G;
                temp_pp.setPixel(x, y, (int) G, (int) G, (int) G, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                G = temp_pp.getPixel(x, y);
                temp_pp.setPixel(x, y, (int) (G / max_G) * 255, (int) (G / max_G) * 255,
                        (int) (G / max_G) * 255);
            }
        }
        return temp_pp;
    }

    public ProcessPlane laplacian() {
        int G = 0;
        int max_G = 0;
        int z2 = 0;
        int z4 = 0;
        int z5 = 0;
        int z6 = 0;
        int z8 = 0;

        //	The formula to compute G of a center pixel in a 3x3 region
        //	G = 4z5 - (z2 + z4 + z6 + z8), where the 3x3 region is
        //
        //	z1	z2	z3
        //	z4	z5	z6
        //	z7	z8	z9

        //	The mask used to compute G at a center point of a 3x3 region
        //
        //	 0	-1	 0
        //	-1	 4	-1
        //	 0	-1	 0

        ProcessPlane temp_pp = new ProcessPlane(getWidth(), getHeight());
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                temp_pp.setPixel(x, y, 0, 0, 0, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                if ((x > 0) && (x < getWidth() - 1) && (y > 0) && (y < getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z2 - z4 - z6 - z8;
                } else if ((x == 0) && (y == 0)) {
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z6 - z8;
                } else if ((x == 0) && (y > 0) && (y < getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z2 - z6 - z8;
                } else if ((x == 0) && (y == getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    G = 4 * z5 - z2 - z6;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    G = 4 * z5 - z2 - z4 - z6;
                } else if ((x == getWidth() - 1) && (y == getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    G = 4 * z5 - z2 - z4;
                } else if ((x == getWidth() - 1) && (y > 0) && (y < getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z2 - z4 - z8;
                } else if ((x == getWidth() - 1) && (y == 0)) {
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z4 - z8;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == 0)) {
                    z4 = getPixel(x - 1, y);
                    z5 = getPixel(x, y);
                    z6 = getPixel(x + 1, y);
                    z8 = getPixel(x, y + 1);
                    G = 4 * z5 - z4 - z6 - z8;
                }
                if (G > max_G)
                    max_G = G;
                temp_pp.setPixel(x, y, (int) G, (int) G, (int) G, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                G = temp_pp.getPixel(x, y);
                temp_pp.setPixel(x, y, (int) (G / max_G) * 255, (int) (G / max_G) * 255,
                        (int) (G / max_G) * 255);
            }
        }
        return temp_pp;
    }

    public ProcessPlane prewitt() {
        int Gx = 0;
        int Gy = 0;
        int G = 0;
        int max_G = 0;
        int z1 = 0;
        int z2 = 0;
        int z3 = 0;
        int z4 = 0;
        int z6 = 0;
        int z7 = 0;
        int z8 = 0;
        int z9 = 0;

        //	The mask used to compute Gx at a center point of a 3x3 region
        //
        //	-1	-1	-1			z1	z2	z3
        //	 0	 0	 0			z4	z5	z6
        //	 1	 1	 1 ,where	z7	z8	z9

        //	The mask used to compute Gy at a center point of a 3x3 region
        //
        //	-1	0	1			z1	z2	z3
        //	-1	0	1			z4	z5	z6
        //	-1	0	1 ,where	z7	z8	z9

        // To approximate the gradient G of a pixel
        //
        //	Gx = (z7 + z8 = z9) - (z1 + z2 + z3)
        //	Gy = (z3 + z6 + z9) - (z1 + z4 + z7)
        //	G = |Gx| + |Gy|

        ProcessPlane temp_pp = new ProcessPlane(getWidth(), getHeight());
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                temp_pp.setPixel(x, y, 0, 0, 0, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                if ((x > 0) && (x < getWidth() - 1) && (y > 0) && (y < getHeight() - 1)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z4 = getPixel(x - 1, y);
                    z6 = getPixel(x + 1, y);
                    Gx = z7 + z8 + z9 - z1 - z2 - z3;
                    Gy = z3 + z6 + z9 - z1 - z4 - z7;
                } else if ((x == 0) && (y == 0)) {
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z6 = getPixel(x + 1, y);
                    Gx = z8 + z9;
                    Gy = z6 + z9;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z6 = getPixel(x + 1, y);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + z8 + z9;
                    Gy = z6 + z9 - z4 - z7;
                } else if ((x == getWidth() - 1) && (y == 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + z8;
                    Gy = -z4 - z7;
                } else if ((x == 0) && (y > 0) && (y < getHeight() - 1)) {
                    z8 = getPixel(x, y + 1);
                    z9 = getPixel(x + 1, y + 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    Gx = z8 + z9 - z2 - z3;
                    Gy = z3 + z6 + z9;
                } else if ((x == 0) && (y == getHeight() - 1)) {
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    Gx = -z2 - z3;
                    Gy = z3 + z6;
                } else if ((x > 0) && (x < getWidth() - 1) && (y == getHeight() - 1)) {
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z3 = getPixel(x + 1, y - 1);
                    z6 = getPixel(x + 1, y);
                    z4 = getPixel(x - 1, y);
                    Gx = -z1 - z2 - z3;
                    Gy = z3 + z6 - z1 - z4;
                } else if ((x == getWidth() - 1) && (y == getHeight() - 1)) {
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    Gx = -z1 - z2;
                    Gy = -z1 - z4;
                } else if ((x == getWidth() - 1) && (y < getHeight() - 1) && (y > 0)) {
                    z7 = getPixel(x - 1, y + 1);
                    z8 = getPixel(x, y + 1);
                    z1 = getPixel(x - 1, y - 1);
                    z2 = getPixel(x, y - 1);
                    z4 = getPixel(x - 1, y);
                    Gx = z7 + z8 - z1 - z2;
                    Gy = -z1 - z4 - z7;
                }
                G = Math.abs(Gx) + Math.abs(Gy);
                if (G > max_G) ;
                max_G = G;
                temp_pp.setPixel(x, y, (int) G, (int) G, (int) G, 255);
            }
        }

        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                G = temp_pp.getPixel(x, y);
                temp_pp.setPixel(x, y, (int) (G / max_G) * 255, (int) (G / max_G) * 255,
                        (int) (G / max_G) * 255);
            }
        }
        return temp_pp;
    }

    public void convolve(double mask[] []) {
        double r, g, b;
        int i,j, ix, jy,a;

        ProcessPlane pp = new ProcessPlane(getWidth(), getHeight());
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                a = getAlpha(x, y);
                r = 0;
                g = 0;
                b = 0;
                for (i = 0; i < mask.length; i++) {
                    for (j = 0; j < mask[0].length; j++) {
                        ix = x + i;
                        jy = y + j;
                        r += mask[i][j] * getRed(ix, jy);
                        g += mask[i][j] * getGreen(ix, jy);
                        b += mask[i][j] * getBlue(ix, jy);
                    }
                }
                pp.setPixel(x, y,
                        (int) r,
                        (int) g,
                        (int) b, a);
            }
        }
        pels = pp.pels;
    }

    public void normalize(double mask[][]) {
        double sum = 0;
        int i, j;
        for (i = 0; i < mask.length; i++)
            for (j = 0; j < mask[0].length; j++)
                sum += mask[i][j];
        for (i = 0; i < mask.length; i++)
            for (j = 0; j < mask[0].length; j++)
                mask[i][j] /= sum;
        System.out.println(mask);
    }


    public void blur2() {
        double mask[][] = {
            {1, 1},
            {1, 1}};
        Mat.scale(mask, 0.25);
        Mat.print(mask);
        convolve(mask);
    }

    public void ImpulseFilter() {
/*
{This is an impulse filter (all zeros with a 1 in the middle)
 minus a 5x5 average (5x5 1's divided by 25), then scaled so
 the smallest tap is 1 (i.e. times 25).}
*/
        double mask [][] = {
            {-1, -1, -1, -1, -1},
            {-1, -1, -1, -1, -1},
            {-1, -1, 24, -1, -1},
            {-1, -1, -1, -1, -1},
            {-1, -1, -1, -1, -1}
        };
        Mat.scale(mask, 1.0 / 48.0);
        convolve(mask);
    }

    public void Sharpen3() {
// '3x3 Sharpen
        double mask [][] =
                {{-1, -1, -1},
                 {-1, 9, -1},
                 {-1, -1, -1}
                };
        Mat.scale(mask, 1.0 / 9.0);
        convolve(mask);
    }

    public void horizontalSegment() {
        double mask [][] = {
            {0, 0, 0},
            {1, 1, 1},
            {0, 0, 0}
        };
        Mat.normalize(mask);
        convolve(mask);
    }

    public void verticalSegment() {
        double mask [][] = {
            {0, 1, 0},
            {0, 1, 0},
            {0, 1, 0}
        };
        Mat.normalize(mask);
        convolve(mask);
    }

    public void Sharpen5() {
//5x5 sharpen filter
        double mask [][] = {
            {-1, -1, -1, -1, -1},
            {-1, -1, -1, -1, -1},
            {-1, -1, 24, -1, -1},
            {-1, -1, -1, -1, -1},
            {-1, -1, -1, -1, -1}
        };
        Mat.scale(mask, 1.0 / 48.0);
        convolve(mask);
    }

    public void gauss7() {
        // 7x7 gauss filter
        double mask [][] = {

            {1, 1, 2, 2, 2, 1, 1},
            {1, 2, 2, 4, 2, 2, 1},
            {2, 2, 4, 8, 4, 2, 2},
            {2, 4, 8, 16, 8, 4, 2},
            {2, 2, 4, 8, 4, 2, 2},
            {1, 2, 2, 4, 2, 2, 1},
            {1, 1, 2, 2, 2, 1, 1}
        };
        Mat.normalize(mask);
        Mat.print(mask);
        convolve(mask);
    }

    public void hat13() {
        // Hat13 filter
        double mask [][] = {
            {0, 0, 0, 0, 0, -1, -1, -1, 0, 0, 0, 0, 0},
            {0, 0, 0, -1, -1, -2, -2, -2, -1, -1, 0, 0, 0},
            {0, 0, -2, -2, -3, -3, -4, -3, -3, -2, -2, 0, 0},
            {0, -1, -2, -3, -3, -3, -2, -3, -3, -3, -2, -1, 0},
            {0, -1, -3, -3, -1, 4, 6, 4, -1, -3, -3, -1, 0},
            {-1, -2, -3, -3, 4, 14, 19, 14, 4, -3, -3, -2, -1},
            {-1, -2, -4, -2, 6, 19, 24, 19, 6, -2, -4, -2, -1},
            {-1, -2, -3, -3, 4, 14, 19, 14, 4, -3, -3, -2, -1},
            {0, -1, -3, -3, -1, 4, 6, 4, -1, -3, -3, -1, 0},
            {0, -1, -2, -3, -3, -3, -2, -3, -3, -3, -2, -1, 0},
            {0, 0, -2, -2, -3, -3, -4, -3, -3, -2, -2, 0, 0},
            {0, 0, 0, -1, -1, -2, -2, -2, -1, -1, 0, 0, 0},
            {0, 0, 0, 0, 0, -1, -1, -1, 0, 0, 0, 0, 0}};
        Mat.normalize(mask);
        convolve(mask);
    }

}